@@ -23,6 +23,7 @@
 #include "e-cal-data-model-subscriber.h"
 #include "gcal-application.h"
 #include "gcal-clock.h"
+#include "gcal-context.h"
 #include "gcal-debug.h"
 #include "gcal-month-cell.h"
 #include "gcal-month-popover.h"
@@ -118,10 +119,8 @@ struct _GcalMonthView
   guint               update_grid_id;
 
   /* property */
-  icaltimetype       *date;
-  GcalManager        *manager;
-
-  GcalWeatherService *weather_service;
+  GDateTime          *date;
+  GcalContext        *context;
 
   gboolean            pending_event_allocation;
 };
@@ -156,8 +155,7 @@ enum
 {
   PROP_0,
   PROP_DATE,
-  PROP_MANAGER,
-  PROP_WEATHER_SERVICE,
+  PROP_CONTEXT,
   N_PROPS
 };
 
@@ -310,8 +308,8 @@ calculate_event_cells (GcalMonthView *self,
       start_date = gcal_event_get_date_start (event);
       start_date = all_day ? g_date_time_ref (start_date) : g_date_time_to_local (start_date);
 
-      if (g_date_time_get_year (start_date) == self->date->year &&
-          g_date_time_get_month (start_date) == self->date->month)
+      if (g_date_time_get_year (start_date) == g_date_time_get_year (self->date) &&
+          g_date_time_get_month (start_date) == g_date_time_get_month (self->date))
         {
           first_cell = g_date_time_get_day_of_month (start_date);
         }
@@ -330,12 +328,12 @@ calculate_event_cells (GcalMonthView *self,
       g_autoptr (GDateTime) end_date = NULL;
       gint last_cell;
 
-      last_cell = icaltime_days_in_month (self->date->month, self->date->year);
+      last_cell = gcal_date_time_get_days_in_month (self->date);
       end_date = gcal_event_get_date_end (event);
       end_date = all_day ? g_date_time_ref (end_date) : g_date_time_to_local (end_date);
 
-      if (g_date_time_get_year (end_date) == self->date->year &&
-          g_date_time_get_month (end_date) == self->date->month)
+      if (g_date_time_get_year (end_date) == g_date_time_get_year (self->date) &&
+          g_date_time_get_month (end_date) == g_date_time_get_month (self->date))
         {
           last_cell = g_date_time_get_day_of_month (end_date);
 
@@ -688,8 +686,8 @@ allocate_multiday_events (GcalMonthView *self,
            * some checks and only applies the dates when it's valid.
            */
           dt_start = g_date_time_new (g_date_time_get_timezone (gcal_event_get_date_start (event)),
-                                      self->date->year,
-                                      self->date->month,
+                                      g_date_time_get_year (self->date),
+                                      g_date_time_get_month (self->date),
                                       day,
                                       0, 0, 0);
 
@@ -871,16 +869,18 @@ static GcalWeatherInfo*
 get_weather_info_for_cell (GcalMonthView *self,
                            guint          cell)
 {
+  GcalWeatherService *weather_service;
   GcalMonthCell *first_cell;
   GPtrArray *weather_infos;
   GDateTime *first_dt;
   GDate first;
   guint i;
 
-  if (!self->weather_service)
+  if (!self->date)
     return NULL;
 
-  weather_infos = gcal_weather_service_get_weather_infos (self->weather_service);
+  weather_service = gcal_context_get_weather_service (self->context);
+  weather_infos = gcal_weather_service_get_weather_infos (weather_service);
 
   first_cell = GCAL_MONTH_CELL (self->month_cell[0][0]);
   first_dt = gcal_month_cell_get_date (first_cell);
@@ -931,12 +931,14 @@ update_weather (GcalMonthView *self,
 static gboolean
 update_month_cells (GcalMonthView *self)
 {
-  g_autoptr (GDateTime) dt;
+  g_autoptr (GDateTime) dt = NULL;
   gboolean show_last_row;
   guint row, col;
 
-  show_last_row = g_date_get_days_in_month (self->date->month, self->date->year) + self->days_delay > 35;
-  dt = g_date_time_new_local (self->date->year, self->date->month, 1, 0, 0, 0);
+  show_last_row = gcal_date_time_get_days_in_month (self->date) + self->days_delay > 35;
+  dt = g_date_time_new_local (g_date_time_get_year (self->date),
+                              g_date_time_get_month (self->date),
+                              1, 0, 0, 0);
 
   for (row = 0; row < 6; row++)
     {
@@ -963,7 +965,7 @@ update_month_cells (GcalMonthView *self)
 
           /* Different month */
           different_month = day < self->days_delay ||
-                            day - self->days_delay >= icaltime_days_in_month (self->date->month, self->date->year);
+                            day - self->days_delay >= gcal_date_time_get_days_in_month (self->date);
 
           gcal_month_cell_set_different_month (cell, different_month);
 
@@ -993,15 +995,15 @@ update_month_cells (GcalMonthView *self)
                 selection_end = selection_start;
 
               /* Swap dates if end is before start */
-              if (datetime_compare_date (selection_end, selection_start) < 0)
+              if (gcal_date_time_compare_date (selection_end, selection_start) < 0)
                 {
                   GDateTime *aux = selection_end;
                   selection_end = selection_start;
                   selection_start = aux;
                 }
 
-              selected = datetime_compare_date (gcal_month_cell_get_date (cell), selection_start) >= 0 &&
-                         datetime_compare_date (gcal_month_cell_get_date (cell), selection_end) <= 0;
+              selected = gcal_date_time_compare_date (gcal_month_cell_get_date (cell), selection_start) >= 0 &&
+                         gcal_date_time_compare_date (gcal_month_cell_get_date (cell), selection_end) <= 0;
             }
 
           gcal_month_cell_set_selected (cell, selected);
@@ -1029,9 +1031,9 @@ update_header_labels (GcalMonthView *self)
 {
   gchar year_str[10] = { 0, };
 
-  g_snprintf (year_str, 10, "%d", self->date->year);
+  g_snprintf (year_str, 10, "%d", g_date_time_get_year (self->date));
 
-  gtk_label_set_label (GTK_LABEL (self->month_label), gcal_get_month_name (self->date->month - 1));
+  gtk_label_set_label (GTK_LABEL (self->month_label), gcal_get_month_name (g_date_time_get_month (self->date) - 1));
   gtk_label_set_label (GTK_LABEL (self->year_label), year_str);
 }
 
@@ -1068,7 +1070,9 @@ add_new_event_button_cb (GtkWidget *button,
   gcal_month_popover_popdown (self->overflow_popover);
 
   day = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (self->overflow_popover), "selected-day"));
-  start_date = g_date_time_new_local (self->date->year, self->date->month, day, 0, 0, 0);
+  start_date = g_date_time_new_local (g_date_time_get_year (self->date),
+                                      g_date_time_get_month (self->date),
+                                      day, 0, 0, 0);
 
   g_signal_emit_by_name (GCAL_VIEW (user_data), "create-event-detailed", start_date, NULL);
 
@@ -1127,7 +1131,7 @@ on_weather_service_weather_changed_cb (GcalWeatherService *weather_service,
  * GcalView interface
  */
 
-static icaltimetype*
+static GDateTime*
 gcal_month_view_get_date (GcalView *view)
 {
   GcalMonthView *self = GCAL_MONTH_VIEW (view);
@@ -1136,22 +1140,23 @@ gcal_month_view_get_date (GcalView *view)
 }
 
 static void
-gcal_month_view_set_date (GcalView     *view,
-                          icaltimetype *date)
+gcal_month_view_set_date (GcalView  *view,
+                          GDateTime *date)
 {
+  g_autofree gchar *new_date_string = NULL;
   GcalMonthView *self;
 
   GCAL_ENTRY;
 
   self = GCAL_MONTH_VIEW (view);
 
-  g_clear_pointer (&self->date, g_free);
+  gcal_set_date_time (&self->date, date);
 
-  self->date = gcal_dup_icaltime (date);
-  self->days_delay = (time_day_of_week (1, self->date->month - 1, self->date->year) - self->first_weekday + 7) % 7;
-  self->keyboard_cell = self->days_delay + (self->date->day - 1);
+  self->days_delay = (time_day_of_week (1, g_date_time_get_month (self->date) - 1, g_date_time_get_year (self->date)) - self->first_weekday + 7) % 7;
+  self->keyboard_cell = self->days_delay + (g_date_time_get_day_of_month (self->date) - 1);
 
-  GCAL_TRACE_MSG ("new date: %s", icaltime_as_ical_string (*date));
+  new_date_string = g_date_time_format (date, "%x %X %z");
+  GCAL_TRACE_MSG ("new date: %s", new_date_string);
 
   update_header_labels (self);
   update_month_cells (self);
@@ -1189,6 +1194,51 @@ gcal_month_view_get_children_by_uuid (GcalView              *view,
   return filter_event_list_by_uid_and_modtype (children, mod, uuid);
 }
 
+static void
+gcal_month_view_update_subscription (GcalView *view)
+{
+  g_autoptr (GDateTime) date_start = NULL;
+  g_autoptr (GDateTime) date_end = NULL;
+  GcalMonthView *self;
+  time_t range_start;
+  time_t range_end;
+
+  self = GCAL_MONTH_VIEW (view);
+
+  g_assert (self->date != NULL);
+  date_start = g_date_time_new_local (g_date_time_get_year (self->date),
+                                      g_date_time_get_month (self->date),
+                                      1, 0, 0, 0);
+  range_start = g_date_time_to_unix (date_start);
+
+  date_end = g_date_time_add_months (date_start, 1);
+  range_end = g_date_time_to_unix (date_end);
+
+  gcal_manager_set_subscriber (gcal_context_get_manager (self->context),
+                               E_CAL_DATA_MODEL_SUBSCRIBER (self),
+                               range_start,
+                               range_end);
+}
+
+static GDateTime*
+gcal_month_view_get_next_date (GcalView *view)
+{
+  GcalMonthView *self = GCAL_MONTH_VIEW (view);
+
+  g_assert (self->date != NULL);
+  return g_date_time_add_months (self->date, 1);
+}
+
+
+static GDateTime*
+gcal_month_view_get_previous_date (GcalView *view)
+{
+  GcalMonthView *self = GCAL_MONTH_VIEW (view);
+
+  g_assert (self->date != NULL);
+  return g_date_time_add_months (self->date, -1);
+}
+
 static void
 gcal_view_interface_init (GcalViewInterface *iface)
 {
@@ -1196,6 +1246,9 @@ gcal_view_interface_init (GcalViewInterface *iface)
   iface->set_date = gcal_month_view_set_date;
   iface->clear_marks = gcal_month_view_clear_marks;
   iface->get_children_by_uuid = gcal_month_view_get_children_by_uuid;
+  iface->update_subscription = gcal_month_view_update_subscription;
+  iface->get_next_date = gcal_month_view_get_next_date;
+  iface->get_previous_date = gcal_month_view_get_previous_date;
 }
 
 
@@ -1237,6 +1290,7 @@ gcal_month_view_component_added (ECalDataModelSubscriber *subscriber,
 {
   g_autoptr (GcalEvent) event = NULL;
   GcalMonthView *self;
+  GcalCalendar *calendar;
   GtkWidget *event_widget;
   GError *error;
 
@@ -1244,7 +1298,9 @@ gcal_month_view_component_added (ECalDataModelSubscriber *subscriber,
 
   error = NULL;
   self = GCAL_MONTH_VIEW (subscriber);
-  event = gcal_event_new (e_client_get_source (E_CLIENT (client)), comp, &error);
+  calendar = gcal_manager_get_calendar_from_source (gcal_context_get_manager (self->context),
+                                                    e_client_get_source (E_CLIENT (client)));
+  event = gcal_event_new (calendar, comp, &error);
 
   if (error)
     {
@@ -1253,7 +1309,7 @@ gcal_month_view_component_added (ECalDataModelSubscriber *subscriber,
       GCAL_RETURN ();
     }
 
-  event_widget = gcal_event_widget_new (event);
+  event_widget = gcal_event_widget_new (self->context, event);
   gcal_event_widget_set_read_only (GCAL_EVENT_WIDGET (event_widget), e_client_is_readonly (E_CLIENT (client)));
 
   gtk_widget_show (event_widget);
@@ -1271,6 +1327,7 @@ gcal_month_view_component_modified (ECalDataModelSubscriber *subscriber,
 {
   g_autoptr (GcalEvent) event = NULL;
   GcalMonthView *self;
+  GcalCalendar *calendar;
   GtkWidget *new_widget;
   GError *error;
   GList *l;
@@ -1279,7 +1336,9 @@ gcal_month_view_component_modified (ECalDataModelSubscriber *subscriber,
 
   error = NULL;
   self = GCAL_MONTH_VIEW (subscriber);
-  event = gcal_event_new (e_client_get_source (E_CLIENT (client)), comp, &error);
+  calendar = gcal_manager_get_calendar_from_source (gcal_context_get_manager (self->context),
+                                                    e_client_get_source (E_CLIENT (client)));
+  event = gcal_event_new (calendar, comp, &error);
 
   if (error)
     {
@@ -1288,7 +1347,7 @@ gcal_month_view_component_modified (ECalDataModelSubscriber *subscriber,
       GCAL_RETURN ();
     }
 
-  new_widget = gcal_event_widget_new (event);
+  new_widget = gcal_event_widget_new (self->context, event);
 
   l = g_hash_table_lookup (self->children, gcal_event_get_uid (event));
 
@@ -1780,7 +1839,7 @@ gcal_month_view_button_press (GtkWidget      *widget,
   GCAL_ENTRY;
 
   self = GCAL_MONTH_VIEW (widget);
-  days = self->days_delay + icaltime_days_in_month (self->date->month, self->date->year);
+  days = self->days_delay + gcal_date_time_get_days_in_month (self->date);
 
   /* The event may have come from a child widget, so make it relative to the month view */
   if (!gcal_translate_child_window_position (widget, event->window, event->x, event->y, &x, &y))
@@ -1793,7 +1852,8 @@ gcal_month_view_button_press (GtkWidget      *widget,
       g_clear_pointer (&self->start_mark_cell, g_date_time_unref);
 
       self->keyboard_cell = clicked_cell;
-      self->start_mark_cell = g_date_time_new_local (self->date->year, self->date->month,
+      self->start_mark_cell = g_date_time_new_local (g_date_time_get_year (self->date),
+                                                     g_date_time_get_month (self->date),
                                                      self->keyboard_cell - self->days_delay + 1,
                                                      0, 0, 0);
 
@@ -1815,7 +1875,7 @@ gcal_month_view_motion_notify_event (GtkWidget      *widget,
   GCAL_ENTRY;
 
   self = GCAL_MONTH_VIEW (widget);
-  days = self->days_delay + icaltime_days_in_month (self->date->month, self->date->year);
+  days = self->days_delay + gcal_date_time_get_days_in_month (self->date);
 
   if (!gcal_translate_child_window_position (widget, event->window, event->x, event->y, &x, &y))
     GCAL_RETURN (GDK_EVENT_PROPAGATE);
@@ -1834,8 +1894,8 @@ gcal_month_view_motion_notify_event (GtkWidget      *widget,
       self->keyboard_cell = new_end_cell;
 
       g_clear_pointer (&self->end_mark_cell, g_date_time_unref);
-      self->end_mark_cell = g_date_time_new_local (self->date->year,
-                                                   self->date->month,
+      self->end_mark_cell = g_date_time_new_local (g_date_time_get_year (self->date),
+                                                   g_date_time_get_month (self->date),
                                                    new_end_cell - self->days_delay + 1,
                                                    0, 0, 0);
 
@@ -1863,7 +1923,7 @@ gcal_month_view_button_release (GtkWidget      *widget,
   GCAL_ENTRY;
 
   self = GCAL_MONTH_VIEW (widget);
-  days = self->days_delay + icaltime_days_in_month (self->date->month, self->date->year);
+  days = self->days_delay + gcal_date_time_get_days_in_month (self->date);
 
   if (!gcal_translate_child_window_position (widget, event->window, event->x, event->y, &x, &y))
     GCAL_RETURN (GDK_EVENT_PROPAGATE);
@@ -1872,14 +1932,17 @@ gcal_month_view_button_release (GtkWidget      *widget,
 
   if (current_day >= self->days_delay && current_day < days)
     {
+      g_autoptr (GDateTime) new_active_date = NULL;
       gboolean valid;
 
-      g_clear_pointer (&self->end_mark_cell, g_date_time_unref);
-
       self->keyboard_cell = current_day;
-      self->end_mark_cell = g_date_time_new_local (self->date->year, self->date->month, current_day - self->days_delay + 1, 0, 0, 0);
+      new_active_date = g_date_time_new_local (g_date_time_get_year (self->date),
+                                               g_date_time_get_month (self->date),
+                                               current_day - self->days_delay + 1,
+                                               0, 0, 0);
 
-      self->date->day = g_date_time_get_day_of_month (self->end_mark_cell);
+      gcal_set_date_time (&self->end_mark_cell, new_active_date);
+      gcal_set_date_time (&self->date, new_active_date);
 
       /* First, make sure to show the popover */
       valid = emit_create_event (self);
@@ -1922,6 +1985,7 @@ gcal_month_view_key_press (GtkWidget   *widget,
   gboolean selection;
   gboolean valid_key;
   gboolean is_ltr;
+  gint days_in_month;
   gint min, max, diff, month_change, current_day;
   gint row, col;
 
@@ -1933,9 +1997,10 @@ gcal_month_view_key_press (GtkWidget   *widget,
   valid_key = FALSE;
   diff = 0;
   month_change = 0;
+  days_in_month = gcal_date_time_get_days_in_month (self->date);
   current_day = self->keyboard_cell - self->days_delay + 1;
   min = self->days_delay;
-  max = self->days_delay + icaltime_days_in_month (self->date->month, self->date->year) - 1;
+  max = self->days_delay + days_in_month - 1;
   is_ltr = gtk_widget_get_direction (widget) != GTK_TEXT_DIR_RTL;
 
   /*
@@ -1944,7 +2009,12 @@ gcal_month_view_key_press (GtkWidget   *widget,
    * focused cell.
    */
   if (selection && self->start_mark_cell == NULL)
-      self->start_mark_cell = g_date_time_new_local (self->date->year, self->date->month, current_day, 0, 0, 0);
+    {
+      self->start_mark_cell = g_date_time_new_local (g_date_time_get_year (self->date),
+                                                     g_date_time_get_month (self->date),
+                                                     current_day,
+                                                     0, 0, 0);
+    }
 
   switch (event->keyval)
     {
@@ -1974,7 +2044,16 @@ gcal_month_view_key_press (GtkWidget   *widget,
        * simulate it by changing the start & end selected cells = keyboard cell.
        */
       if (!selection && !self->start_mark_cell && !self->end_mark_cell)
-        self->start_mark_cell = self->end_mark_cell = g_date_time_new_local (self->date->year, self->date->month, current_day, 0, 0, 0);
+        {
+          g_autoptr (GDateTime) new_mark = NULL;
+
+          new_mark = g_date_time_new_local (g_date_time_get_year (self->date),
+                                            g_date_time_get_month (self->date),
+                                            current_day,
+                                            0, 0, 0);
+          self->start_mark_cell = g_object_ref (new_mark);
+          self->end_mark_cell = g_object_ref (new_mark);
+        }
 
       create_event = TRUE;
       break;
@@ -1993,11 +2072,12 @@ gcal_month_view_key_press (GtkWidget   *widget,
     }
   else
     {
-      month_change = self->keyboard_cell + diff > max ? 1 : -1;
-      self->date->month += month_change;
-      *self->date = icaltime_normalize (*self->date);
+      g_autoptr (GDateTime) new_month = NULL;
 
-      self->days_delay = (time_day_of_week (1, self->date->month - 1, self->date->year) - self->first_weekday + 7) % 7;
+      month_change = self->keyboard_cell + diff > max ? 1 : -1;
+      new_month = g_date_time_add_months (self->date, month_change);
+
+      self->days_delay = (time_day_of_week (1, g_date_time_get_month (new_month) - 1, g_date_time_get_year (new_month)) - self->first_weekday + 7) % 7;
 
       /*
        * Set keyboard cell value to the sum or difference of days delay of successive
@@ -2009,7 +2089,7 @@ gcal_month_view_key_press (GtkWidget   *widget,
       if (month_change == 1)
         self->keyboard_cell = self->days_delay + self->keyboard_cell + diff - max - 1;
       else
-        self->keyboard_cell = self->days_delay + icaltime_days_in_month (self->date->month, self->date->year) - min + self->keyboard_cell + diff;
+        self->keyboard_cell = self->days_delay + gcal_date_time_get_days_in_month (new_month) - min + self->keyboard_cell + diff;
     }
 
   /* Focus the selected month cell */
@@ -2019,7 +2099,7 @@ gcal_month_view_key_press (GtkWidget   *widget,
   gtk_widget_grab_focus (self->month_cell[row][col]);
 
   current_day = self->keyboard_cell - self->days_delay + 1;
-  self->date->day = current_day;
+  //self->date->day = current_day;
 
   /*
    * We can only emit the :create-event signal ~after~ grabbing the focus, otherwise
@@ -2032,7 +2112,10 @@ gcal_month_view_key_press (GtkWidget   *widget,
 
   if (selection)
     {
-      self->end_mark_cell = g_date_time_new_local (self->date->year, self->date->month, current_day, 0, 0, 0);
+      self->end_mark_cell = g_date_time_new_local (g_date_time_get_year (self->date),
+                                                   g_date_time_get_month (self->date),
+                                                   current_day,
+                                                   0, 0, 0);
     }
   else if (!selection && valid_key)
     {
@@ -2055,8 +2138,14 @@ gcal_month_view_scroll_event (GtkWidget      *widget,
    */
   if (should_change_date_for_scroll (&self->scroll_value, scroll_event))
     {
-      self->date->month += self->scroll_value > 0 ? 1 : -1;
-      *self->date = icaltime_normalize (*self->date);
+      g_autoptr (GDateTime) new_date = NULL;
+      gint diff;
+
+      diff = self->scroll_value > 0 ? 1 : -1;
+      new_date = g_date_time_add_months (self->date, diff);
+
+      gcal_clear_date_time (&self->date);
+      self->date = g_steal_pointer (&new_date);
       self->scroll_value = 0;
 
       gtk_widget_queue_draw (widget);
@@ -2087,21 +2176,26 @@ gcal_month_view_set_property (GObject       *object,
       gcal_view_set_date (GCAL_VIEW (object), g_value_get_boxed (value));
       break;
 
-    case PROP_MANAGER:
-      self->manager = g_value_dup_object (value);
+    case PROP_CONTEXT:
+      self->context = g_value_dup_object (value);
 
       for (i = 0; i < 42; i++)
-        gcal_month_cell_set_manager (GCAL_MONTH_CELL (self->month_cell[i / 7][i % 7]), self->manager);
+        gcal_month_cell_set_context (GCAL_MONTH_CELL (self->month_cell[i / 7][i % 7]), self->context);
 
-      g_object_notify (object, "manager");
-      break;
+      g_signal_connect_object (gcal_context_get_clock (self->context),
+                               "day-changed",
+                               G_CALLBACK (update_month_cells),
+                               self,
+                               G_CONNECT_SWAPPED);
 
-    case PROP_WEATHER_SERVICE:
-      gcal_view_set_weather_service_impl_helper (&self->weather_service,
-                                                 g_value_get_object (value),
-                                                 (GcalWeatherUpdateFunc) update_weather,
-                                                 G_CALLBACK (on_weather_service_weather_changed_cb),
-                                                 GTK_WIDGET (self));
+      g_signal_connect_object (gcal_context_get_weather_service (self->context),
+                               "weather-changed",
+                               G_CALLBACK (on_weather_service_weather_changed_cb),
+                               self,
+                               0);
+      update_weather (self, TRUE);
+
+      g_object_notify (object, "context");
       break;
 
     default:
@@ -2124,12 +2218,8 @@ gcal_month_view_get_property (GObject       *object,
       g_value_set_boxed (value, self->date);
       break;
 
-    case PROP_MANAGER:
-      g_value_set_object (value, self->manager);
-      break;
-
-    case PROP_WEATHER_SERVICE:
-      g_value_set_boxed (value, self->weather_service);
+    case PROP_CONTEXT:
+      g_value_set_object (value, self->context);
       break;
 
     default:
@@ -2143,14 +2233,13 @@ gcal_month_view_finalize (GObject *object)
 {
   GcalMonthView *self = GCAL_MONTH_VIEW (object);
 
-  g_clear_pointer (&self->date, g_free);
+  gcal_clear_date_time (&self->date);
   g_clear_pointer (&self->children, g_hash_table_destroy);
   g_clear_pointer (&self->single_cell_children, g_hash_table_destroy);
   g_clear_pointer (&self->overflow_cells, g_hash_table_destroy);
   g_clear_pointer (&self->multi_cell_children, g_list_free);
 
-  g_clear_object (&self->manager);
-  g_clear_object (&self->weather_service);
+  g_clear_object (&self->context);
 
   if (self->update_grid_id > 0)
     {
@@ -2193,8 +2282,7 @@ gcal_month_view_class_init (GcalMonthViewClass *klass)
   container_class->forall = gcal_month_view_forall;
 
   g_object_class_override_property (object_class, PROP_DATE, "active-date");
-  g_object_class_override_property (object_class, PROP_MANAGER, "manager");
-  g_object_class_override_property (object_class, PROP_WEATHER_SERVICE, "weather-service");
+  g_object_class_override_property (object_class, PROP_CONTEXT, "context");
 
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/calendar/month-view.ui");
 
@@ -2218,13 +2306,10 @@ gcal_month_view_class_init (GcalMonthViewClass *klass)
 static void
 gcal_month_view_init (GcalMonthView *self)
 {
-  GcalApplication *application;
-
   gtk_widget_init_template (GTK_WIDGET (self));
 
   gtk_widget_set_has_window (GTK_WIDGET (self), FALSE);
 
-  self->weather_service = NULL;
   self->children = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, (GDestroyNotify) g_list_free);
   self->single_cell_children = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, (GDestroyNotify) g_list_free);
   self->overflow_cells = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, (GDestroyNotify) g_list_free);
@@ -2248,23 +2333,11 @@ gcal_month_view_init (GcalMonthView *self)
   self->overflow_popover = (GcalMonthPopover*) gcal_month_popover_new ();
 
   g_object_bind_property (self,
-                          "manager",
+                          "context",
                           self->overflow_popover,
-                          "manager",
+                          "context",
                           G_BINDING_DEFAULT);
 
   g_signal_connect (self->overflow_popover, "event-activated", G_CALLBACK (on_month_popover_event_activated_cb), self);
-
-
-  /* Connect to the wall clock */
-  application = GCAL_APPLICATION (g_application_get_default ());
-
-  g_assert (application != NULL);
-
-  g_signal_connect_object (gcal_application_get_clock (application),
-                           "day-changed",
-                           G_CALLBACK (update_month_cells),
-                           self,
-                           G_CONNECT_SWAPPED);
 }
 
