@@ -22,6 +22,7 @@
 #define G_LOG_DOMAIN "GcalWeekHeader"
 
 #include "gcal-application.h"
+#include "gcal-context.h"
 #include "gcal-clock.h"
 #include "gcal-debug.h"
 #include "gcal-event-widget.h"
@@ -70,7 +71,7 @@ struct _GcalWeekHeader
   GtkWidget          *expand_button_image;
   GtkWidget          *header_labels_box;
 
-  GcalManager        *manager;
+  GcalContext        *context;
 
   /*
    * Stores the events as they come from the week-view
@@ -88,7 +89,7 @@ struct _GcalWeekHeader
    */
   gboolean            expanded;
 
-  icaltimetype       *active_date;
+  GDateTime          *active_date;
 
   gint                selection_start;
   gint                selection_end;
@@ -107,13 +108,6 @@ typedef enum
   DOWN
 } MoveDirection;
 
-enum
-{
-  PROP_0,
-  PROP_WEATHER_SERVICE,
-  NUM_PROPS
-};
-
 enum
 {
   EVENT_ACTIVATED,
@@ -144,6 +138,77 @@ wid_clear (WeatherInfoDay *wid)
     g_clear_object (&wid->icon_buf);
 }
 
+static gint
+add_weather_infos (GcalWeekHeader *self,
+                   GPtrArray      *weather_infos)
+{
+  g_autoptr (GDateTime) week_start_dt = NULL;
+  GDate week_start;
+  gint consumed = 0;
+  guint i;
+
+  if (!self->active_date)
+    return 0;
+
+  week_start_dt = gcal_date_time_get_start_of_week (self->active_date);
+  g_date_set_dmy (&week_start,
+                  g_date_time_get_day_of_month (week_start_dt),
+                  g_date_time_get_month (week_start_dt),
+                  g_date_time_get_year (week_start_dt));
+
+  for (i = 0; weather_infos && i < weather_infos->len; i++)
+    {
+      GcalWeatherInfo *gwi; /* unowned */
+      GDate gwi_date;
+      gint day_diff;
+
+      gwi = g_ptr_array_index (weather_infos, i);
+
+      gcal_weather_info_get_date (gwi, &gwi_date);
+
+      day_diff = g_date_days_between (&week_start, &gwi_date);
+      if (day_diff >= 0 && day_diff < G_N_ELEMENTS (self->weather_infos))
+        {
+          wid_clear (&self->weather_infos[day_diff]);
+          self->weather_infos[day_diff].winfo = g_object_ref (gwi);
+          consumed++;
+        }
+    }
+
+  if (consumed > 0)
+    gtk_widget_queue_draw (GTK_WIDGET (self));
+
+  return consumed;
+}
+
+static void
+clear_weather_infos (GcalWeekHeader *self)
+{
+  g_return_if_fail (GCAL_IS_WEEK_HEADER (self));
+
+  for (gint i = 0; i < G_N_ELEMENTS (self->weather_infos); i++)
+    wid_clear (&self->weather_infos[i]);
+
+  gtk_widget_queue_draw (GTK_WIDGET (self));
+}
+
+static void
+update_weather_infos (GcalWeekHeader *self)
+{
+  GPtrArray* weather_infos;
+
+  g_return_if_fail (GCAL_IS_WEEK_HEADER (self));
+
+  clear_weather_infos (self);
+
+  if (!self->weather_service)
+    return;
+
+  weather_infos = gcal_weather_service_get_weather_infos (self->weather_service);
+  add_weather_infos (self, weather_infos);
+}
+
+
 /* Event activation methods */
 static void
 on_event_widget_activated (GcalEventWidget *widget,
@@ -255,7 +320,7 @@ on_button_released (GcalWeekHeader *self,
       start = start - end;
     }
 
-  week_start = get_start_of_week (self->active_date);
+  week_start = gcal_date_time_get_start_of_week (self->active_date);
   selection_start = g_date_time_add_days (week_start, start);
   selection_end = end == start ? g_date_time_ref (selection_start) : g_date_time_add_days (week_start, end + 1);
 
@@ -286,11 +351,11 @@ static void
 on_weather_update (GcalWeatherService *weather_service,
                    GcalWeekHeader     *self)
 {
-  g_return_if_fail (GCAL_IS_WEATHER_SERVICE (weather_service));
-  g_return_if_fail (GCAL_IS_WEEK_HEADER (self));
-  g_return_if_fail (self->weather_service == weather_service);
+  g_assert (GCAL_IS_WEATHER_SERVICE (weather_service));
+  g_assert (GCAL_IS_WEEK_HEADER (self));
+  g_assert (self->weather_service == weather_service);
 
-  gcal_week_header_update_weather_infos (self);
+  update_weather_infos (self);
 }
 
 static GcalEvent*
@@ -320,7 +385,7 @@ get_today_column (GcalWeekHeader *self)
   gint days_diff;
 
   today = g_date_time_new_now_local ();
-  week_start = get_start_of_week (self->active_date);
+  week_start = gcal_date_time_get_start_of_week (self->active_date);
   days_diff = g_date_time_difference (today, week_start) / G_TIME_SPAN_DAY;
 
   /* Today is out of range */
@@ -565,7 +630,7 @@ split_event_widget_at_column (GcalWeekHeader *self,
   gint new_width;
   gint old_width;
 
-  week_start = get_start_of_week (self->active_date);
+  week_start = gcal_date_time_get_start_of_week (self->active_date);
   column_date = g_date_time_add_days (week_start, column);
   end_column_date = g_date_time_add_days (column_date, 1);
 
@@ -734,7 +799,7 @@ add_event_to_grid (GcalWeekHeader *self,
   move_events_at_column (self, DOWN, start, position);
 
   /* Add the event to the grid */
-  widget = gcal_event_widget_new (event);
+  widget = gcal_event_widget_new (self->context, event);
   setup_event_widget (self, widget);
 
   gtk_grid_attach (GTK_GRID (self->grid),
@@ -756,8 +821,8 @@ add_event_to_grid (GcalWeekHeader *self,
   if (!gcal_event_is_multiday (event))
     return;
 
-  week_start = get_start_of_week (self->active_date);
-  week_end = get_end_of_week (self->active_date);
+  week_start = gcal_date_time_get_start_of_week (self->active_date);
+  week_end = gcal_date_time_get_end_of_week (self->active_date);
 
   gcal_event_widget_set_date_start (GCAL_EVENT_WIDGET (widget), week_start);
   gcal_event_widget_set_date_end (GCAL_EVENT_WIDGET (widget), week_end);
@@ -824,7 +889,7 @@ add_event_to_grid (GcalWeekHeader *self,
 
 static void
 update_unchanged_events (GcalWeekHeader *self,
-                         icaltimetype   *new_icaldt)
+                         GDateTime      *new_date)
 {
   g_autoptr (GDateTime) new_week_start, new_week_end;
   g_autoptr (GDateTime) utc_week_start, utc_week_end;
@@ -833,8 +898,8 @@ update_unchanged_events (GcalWeekHeader *self,
 
   events_to_update = NULL;
 
-  new_week_start = get_start_of_week (new_icaldt);
-  new_week_end = get_end_of_week (new_icaldt);
+  new_week_start = gcal_date_time_get_start_of_week (new_date);
+  new_week_end = gcal_date_time_get_end_of_week (new_date);
 
   utc_week_start = g_date_time_new_utc (g_date_time_get_year (new_week_start),
                                         g_date_time_get_month (new_week_start),
@@ -907,7 +972,7 @@ update_title (GcalWeekHeader *self)
   if(!self->active_date)
     return;
 
-  week_start = get_start_of_week (self->active_date);
+  week_start = gcal_date_time_get_start_of_week (self->active_date);
   week_end = g_date_time_add_days (week_start, 6);
   week_mid = g_date_time_add_days (week_start, 3);
 
@@ -1092,7 +1157,7 @@ gcal_week_header_finalize (GObject *object)
   GcalWeekHeader *self = GCAL_WEEK_HEADER (object);
   gint i;
 
-  g_clear_pointer (&self->active_date, g_free);
+  gcal_clear_date_time (&self->active_date);
 
   for (i = 0; i < 7; i++)
     g_list_free (self->events[i]);
@@ -1103,46 +1168,6 @@ gcal_week_header_finalize (GObject *object)
     wid_clear (&self->weather_infos[i]);
 }
 
-static void
-gcal_week_header_get_property (GObject    *object,
-                               guint       property_id,
-                               GValue     *value,
-                               GParamSpec *pspec)
-{
-  GcalWeekHeader *self = GCAL_WEEK_HEADER (object);
-
-  switch (property_id)
-    {
-    case PROP_WEATHER_SERVICE:
-      g_value_set_boxed (value, self->weather_service);
-      break;
-
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
-      break;
-    }
-}
-
-static void
-gcal_week_header_set_property (GObject      *object,
-                               guint         property_id,
-                               const GValue *value,
-                               GParamSpec   *pspec)
-{
-  GcalWeekHeader *self = GCAL_WEEK_HEADER (object);
-
-  switch (property_id)
-    {
-    case PROP_WEATHER_SERVICE:
-        gcal_week_header_set_weather_service (self, GCAL_WEATHER_SERVICE (g_value_get_object (value)));
-        break;
-
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
-      break;
-    }
-}
-
 static void
 gcal_week_header_size_allocate (GtkWidget     *widget,
                                 GtkAllocation *alloc)
@@ -1204,9 +1229,9 @@ gcal_week_header_draw (GtkWidget      *widget,
   pango_font_description_set_weight (bold_font, PANGO_WEIGHT_MEDIUM);
   pango_layout_set_font_description (layout, bold_font);
 
-  week_start = get_start_of_week (self->active_date);
+  week_start = gcal_date_time_get_start_of_week (self->active_date);
   week_end = g_date_time_add_days (week_start, 6);
-  current_cell = icaltime_day_of_week (*(self->active_date)) - 1;
+  current_cell = g_date_time_get_day_of_week (self->active_date) - 1;
   current_cell = (7 + current_cell - self->first_weekday) % 7;
   today_column = get_today_column (self);
 
@@ -1270,6 +1295,7 @@ gcal_week_header_draw (GtkWidget      *widget,
 
   for (i = 0; i < 7; i++)
     {
+      g_autoptr (GDateTime) day = NULL;
       WeatherInfoDay *wdinfo; /* unowned */
       gchar *weekday_date, *weekday_abv, *weekday;
       gdouble x;
@@ -1277,7 +1303,8 @@ gcal_week_header_draw (GtkWidget      *widget,
       gint font_width;
       gint n_day;
 
-      n_day = g_date_time_get_day_of_month (week_start) + i;
+      day = g_date_time_add_days (week_start, i);
+      n_day = g_date_time_get_day_of_month (day);
 
       if (n_day > g_date_get_days_in_month (g_date_time_get_month (week_start), g_date_time_get_year (week_start)))
         n_day = n_day - g_date_get_days_in_month (g_date_time_get_month (week_start), g_date_time_get_year (week_start));
@@ -1312,8 +1339,8 @@ gcal_week_header_draw (GtkWidget      *widget,
       gtk_style_context_restore (context);
 
       /* Draws the days name */
-      weekday = g_utf8_strup (gcal_get_weekday ((i + self->first_weekday) % 7), -1);
-      weekday_abv = g_strdup (weekday);
+      weekday = g_date_time_format (day, "%a");
+      weekday_abv = g_utf8_strup (weekday, -1);
       g_free (weekday);
 
       gtk_style_context_save (context);
@@ -1537,7 +1564,7 @@ gcal_week_header_drag_drop (GtkWidget      *widget,
   event = gcal_event_widget_get_event (GCAL_EVENT_WIDGET (event_widget));
   start_date = gcal_event_get_date_start (event);
   end_date = gcal_event_get_date_end (event);
-  week_start = get_start_of_week (self->active_date);
+  week_start = gcal_date_time_get_start_of_week (self->active_date);
 
   turn_all_day = !gcal_event_is_multiday (event) || gcal_event_get_all_day (event);
 
@@ -1580,7 +1607,9 @@ gcal_week_header_drag_drop (GtkWidget      *widget,
     gcal_event_set_all_day (event, TRUE);
 
   /* Commit the changes */
-  gcal_manager_update_event (self->manager, event, GCAL_RECURRENCE_MOD_THIS_ONLY);
+  gcal_manager_update_event (gcal_context_get_manager (self->context),
+                             event,
+                             GCAL_RECURRENCE_MOD_THIS_ONLY);
 
   /* Cancel the DnD */
   self->dnd_cell = -1;
@@ -1612,8 +1641,6 @@ gcal_week_header_class_init (GcalWeekHeaderClass *kclass)
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (kclass);
 
   object_class->finalize = gcal_week_header_finalize;
-  object_class->get_property = gcal_week_header_get_property;
-  object_class->set_property = gcal_week_header_set_property;
 
   widget_class->draw = gcal_week_header_draw;
   widget_class->size_allocate = gcal_week_header_size_allocate;
@@ -1621,19 +1648,6 @@ gcal_week_header_class_init (GcalWeekHeaderClass *kclass)
   widget_class->drag_leave = gcal_week_header_drag_leave;
   widget_class->drag_drop = gcal_week_header_drag_drop;
 
-  /**
-   * GcalWeekHeader::weather-service:
-   *
-   * The #GcalWeatherService of the view.
-   */
-  g_object_class_install_property (G_OBJECT_CLASS (kclass),
-                                   PROP_WEATHER_SERVICE,
-                                   g_param_spec_object ("weather-service",
-                                                        "The weather service",
-                                                        "The weather service of the view",
-                                                        GCAL_TYPE_WEATHER_SERVICE,
-                                                        G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-
   signals[EVENT_ACTIVATED] = g_signal_new ("event-activated",
                                            GCAL_TYPE_WEEK_HEADER,
                                            G_SIGNAL_RUN_FIRST,
@@ -1666,8 +1680,6 @@ gcal_week_header_class_init (GcalWeekHeaderClass *kclass)
 static void
 gcal_week_header_init (GcalWeekHeader *self)
 {
-  GcalApplication *application;
-
   self->expanded = FALSE;
   self->selection_start = -1;
   self->selection_end = -1;
@@ -1687,147 +1699,29 @@ G_GNUC_END_IGNORE_DEPRECATIONS
                      NULL,
                      0,
                      GDK_ACTION_MOVE);
+}
 
-  /* Connect to the wall clock */
-  application = GCAL_APPLICATION (g_application_get_default ());
+void
+gcal_week_header_set_context (GcalWeekHeader *self,
+                              GcalContext    *context)
+{
+  g_return_if_fail (GCAL_IS_WEEK_HEADER (self));
 
-  g_assert (application != NULL);
+  self->context = context;
 
-  g_signal_connect_object (gcal_application_get_clock (application),
+  g_signal_connect_object (gcal_context_get_clock (context),
                            "day-changed",
                            G_CALLBACK (gtk_widget_queue_draw),
                            self,
                            G_CONNECT_SWAPPED);
-}
 
-/* Private API */
+  g_signal_connect_object (gcal_context_get_weather_service (self->context),
+                           "weather-changed",
+                           G_CALLBACK (on_weather_update),
+                           self,
+                           0);
 
-/* gcal_week_header_add_weather_infos:
- * @self:  (not nullable): The #GcalWeekHeader instance.
- * @winfo: List of #GcalWeatherInfos to add to @self.
- *
- * Adds weather information to this header. @self
- * only adds weather information that can be
- * displayed with current settings.
- *
- * Return: Number of consumed weather information
- *         objects.
- */
-static gint
-gcal_week_header_add_weather_infos (GcalWeekHeader *self,
-                                    GPtrArray      *weather_infos)
-{
-  g_autoptr (GDateTime) week_start_dt = NULL;
-  GDate week_start;
-  gint consumed = 0;
-  guint i;
-
-  if (!self->active_date)
-    return 0;
-
-  week_start_dt = get_start_of_week (self->active_date);
-  g_date_set_dmy (&week_start,
-                  g_date_time_get_day_of_month (week_start_dt),
-                  g_date_time_get_month (week_start_dt),
-                  g_date_time_get_year (week_start_dt));
-
-  for (i = 0; weather_infos && i < weather_infos->len; i++)
-    {
-      GcalWeatherInfo *gwi; /* unowned */
-      GDate gwi_date;
-      gint day_diff;
-
-      gwi = g_ptr_array_index (weather_infos, i);
-
-      gcal_weather_info_get_date (gwi, &gwi_date);
-
-      day_diff = g_date_days_between (&week_start, &gwi_date);
-      if (day_diff >= 0 && day_diff < G_N_ELEMENTS (self->weather_infos))
-        {
-          wid_clear (&self->weather_infos[day_diff]);
-          self->weather_infos[day_diff].winfo = g_object_ref (gwi);
-          consumed++;
-        }
-    }
-
-  if (consumed > 0)
-    gtk_widget_queue_draw (GTK_WIDGET (self));
-
-  return consumed;
-}
-
-/* gcal_week_header_clear_weather_infos:
- * @self: The #GcalWeekHeader instance.
- *
- * Removes all registered weather information objects from
- * this widget.
- */
-static void
-gcal_week_header_clear_weather_infos (GcalWeekHeader *self)
-{
-  g_return_if_fail (GCAL_IS_WEEK_HEADER (self));
-
-  for (gint i = 0; i < G_N_ELEMENTS (self->weather_infos); i++)
-    wid_clear (&self->weather_infos[i]);
-
-  gtk_widget_queue_draw (GTK_WIDGET (self));
-}
-
-
-/* Public API */
-
-/**
- * gcal_week_header_set_weather_service:
- * @self:    The #GcalWeekHeader instance.
- * @service: (nullable): The weather service to query.
- *
- * Note that #GcalWeekHeader does not hold a strong reference
- * to its weather service.
- */
-void
-gcal_week_header_set_weather_service (GcalWeekHeader     *self,
-                                      GcalWeatherService *service)
-{
-  g_return_if_fail (GCAL_IS_WEEK_HEADER (self));
-  g_return_if_fail (service == NULL || GCAL_IS_WEATHER_SERVICE (service));
-
-  gcal_view_set_weather_service_impl_helper (&self->weather_service,
-                                             service,
-                                             (GcalWeatherUpdateFunc) gcal_week_header_update_weather_infos,
-                                             (GCallback) on_weather_update,
-                                             GTK_WIDGET (self));
-}
-
-/**
- * gcal_week_header_update_weather_infos:
- * @self: The #GcalWeekHeader instance.
- *
- * Retrieves latest weather information from registered
- * weather service and displays it.
- */
-void
-gcal_week_header_update_weather_infos (GcalWeekHeader *self)
-{
-  GPtrArray* weather_infos;
-
-  g_return_if_fail (GCAL_IS_WEEK_HEADER (self));
-
-  gcal_week_header_clear_weather_infos (self);
-
-  if (!self->weather_service)
-    return;
-
-  weather_infos = gcal_weather_service_get_weather_infos (self->weather_service);
-  gcal_week_header_add_weather_infos (self, weather_infos);
-}
-
-void
-gcal_week_header_set_manager (GcalWeekHeader *self,
-                              GcalManager    *manager)
-{
-  g_return_if_fail (GCAL_IS_WEEK_HEADER (self));
-
-  self->manager = manager;
+  update_weather_infos (self);
 }
 
 void
@@ -1844,8 +1738,8 @@ gcal_week_header_add_event (GcalWeekHeader *self,
   g_return_if_fail (GCAL_IS_WEEK_HEADER (self));
 
   all_day = gcal_event_get_all_day (event);
-  week_start = get_start_of_week (self->active_date);
-  week_end = get_end_of_week (self->active_date);
+  week_start = gcal_date_time_get_start_of_week (self->active_date);
+  week_end = gcal_date_time_get_end_of_week (self->active_date);
 
   /* Retrieve the real start and end dates */
   if (all_day)
@@ -1870,11 +1764,11 @@ gcal_week_header_add_event (GcalWeekHeader *self,
        */
       aux = week_start;
       week_start = utc_week_start;
-      gcal_clear_datetime (&aux);
+      gcal_clear_date_time (&aux);
 
       aux = week_end;
       week_end = utc_week_end;
-      gcal_clear_datetime (&aux);
+      gcal_clear_date_time (&aux);
 
     }
   else
@@ -1884,7 +1778,7 @@ gcal_week_header_add_event (GcalWeekHeader *self,
     }
 
   /* Start position */
-  if (datetime_compare_date (start_date, week_start) >= 0)
+  if (gcal_date_time_compare_date (start_date, week_start) >= 0)
     start = floor (g_date_time_difference (start_date, week_start) / G_TIME_SPAN_DAY);
   else
     start = 0;
@@ -2011,59 +1905,31 @@ gcal_week_header_clear_marks (GcalWeekHeader *self)
 
 void
 gcal_week_header_set_date (GcalWeekHeader *self,
-                           icaltimetype   *date)
+                           GDateTime      *date)
 {
-  icaltimetype *old_date, *new_date;
-
-  old_date = self->active_date;
-  new_date = gcal_dup_icaltime (date);
+  gboolean had_date;
 
   /*
    * If the active date changed, but we're still in the same week,
    * there's no need to recalculate visible events.
    */
-  if (old_date && new_date &&
-      old_date->year == new_date->year &&
-      icaltime_week_number (*old_date) == icaltime_week_number (*new_date))
+  if (self->active_date && date &&
+      g_date_time_get_year (self->active_date) == g_date_time_get_year (date) &&
+      g_date_time_get_week_of_year (self->active_date) == g_date_time_get_week_of_year (date))
     {
-      g_free (new_date);
       return;
     }
 
-  self->active_date = new_date;
+  had_date = self->active_date != NULL;
+
+  gcal_set_date_time (&self->active_date, date);
 
   update_title (self);
   gtk_widget_queue_draw (GTK_WIDGET (self));
 
-  if (old_date)
+  if (had_date)
     update_unchanged_events (self, self->active_date);
 
-  gcal_week_header_update_weather_infos (self);
-
-  g_clear_pointer (&old_date, g_free);
+  update_weather_infos (self);
 }
 
-/**
- * gcal_week_header_get_weather_infos:
- * @self: The #GcalWeekHeader instance.
- *
- * Returns a list of shown weather informations.
- *
- * @Returns: (transfer container):
- *           A GSList. The callee is responsible for freeing it.
- *           Elements are owned by the widget. Do not modify.
- */
-GSList*
-gcal_week_header_get_shown_weather_infos (GcalWeekHeader *self)
-{
-  g_return_val_if_fail (GCAL_IS_WEEK_HEADER (self), NULL);
-  GSList* lst = NULL; /* owned[unowned] */
-
-  for (int i = 0; i < G_N_ELEMENTS (self->weather_infos); i++)
-    {
-      if (self->weather_infos[i].winfo != NULL)
-        lst = g_slist_prepend (lst, self->weather_infos[i].winfo);
-    }
-
-   return lst;
-}
